@@ -11,8 +11,8 @@
 
 ;; GNU Emacs is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 2, or (at your option)
-;; any later version.
+;; the Free Software Foundation; either version 3 of the License, or
+;; (at your option) any later version.
 
 ;; GNU Emacs is distributed in the hope that it will be useful,
 ;; but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -20,9 +20,8 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-;; Boston, MA 02110-1301, USA.
+;; along with GNU Emacs; see the file COPYING.  If not, see
+;; <http://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
@@ -72,8 +71,18 @@
 
 (defcustom tramp-persistency-file-name
   (cond
+   ;; GNU Emacs.
+   ((and (boundp 'user-emacs-directory)
+	 (stringp (symbol-value 'user-emacs-directory))
+	 (file-directory-p (symbol-value 'user-emacs-directory)))
+    (expand-file-name "tramp" (symbol-value 'user-emacs-directory)))
    ((and (not (featurep 'xemacs)) (file-directory-p "~/.emacs.d/"))
     "~/.emacs.d/tramp")
+   ;; XEmacs.
+   ((and (boundp 'user-init-directory)
+	 (stringp (symbol-value 'user-init-directory))
+	 (file-directory-p (symbol-value 'user-init-directory)))
+    (expand-file-name "tramp" (symbol-value 'user-init-directory)))
    ((and (featurep 'xemacs) (file-directory-p "~/.xemacs/"))
     "~/.xemacs/tramp")
    ;; For users without `~/.emacs.d/' or `~/.xemacs/'.
@@ -87,7 +96,7 @@
 Returns DEFAULT if not set."
   ;; Unify localname.
   (setq vec (copy-sequence vec))
-  (aset vec 4 (directory-file-name file))
+  (aset vec 3 (directory-file-name file))
   (let* ((hash (or (gethash vec tramp-cache-data)
 		   (puthash vec (make-hash-table :test 'equal)
 			    tramp-cache-data)))
@@ -102,7 +111,7 @@ Returns DEFAULT if not set."
 Returns VALUE."
   ;; Unify localname.
   (setq vec (copy-sequence vec))
-  (aset vec 4 (directory-file-name file))
+  (aset vec 3 (directory-file-name file))
   (let ((hash (or (gethash vec tramp-cache-data)
 		  (puthash vec (make-hash-table :test 'equal)
 			   tramp-cache-data))))
@@ -114,7 +123,7 @@ Returns VALUE."
   "Remove all properties of FILE in the cache context of VEC."
   ;; Unify localname.
   (setq vec (copy-sequence vec))
-  (aset vec 4 (directory-file-name file))
+  (aset vec 3 (directory-file-name file))
   (tramp-message vec 8 "%s" file)
   (remhash vec tramp-cache-data))
 
@@ -179,7 +188,7 @@ If the value is not set for the connection, returns DEFAULT."
   ;; order to avoid side effects.
   (when (vectorp key)
     (setq key (copy-sequence key))
-    (aset key 4 nil))
+    (aset key 3 nil))
   (let* ((hash (gethash key tramp-cache-data))
 	 (value (if (hash-table-p hash)
 		   (gethash property hash default)
@@ -195,7 +204,7 @@ PROPERTY is set persistent when KEY is a vector."
   ;; order to avoid side effects.
   (when (vectorp key)
     (setq key (copy-sequence key))
-    (aset key 4 nil))
+    (aset key 3 nil))
   (let ((hash (or (gethash key tramp-cache-data)
 		  (puthash key (make-hash-table :test 'equal)
 			    tramp-cache-data))))
@@ -217,7 +226,7 @@ function is intended to run also as process sentinel."
   ;; order to avoid side effects.
   (when (vectorp key)
     (setq key (copy-sequence key))
-    (aset key 4 nil))
+    (aset key 3 nil))
 ;  (tramp-message key 7 "%s" event)
   (remhash key tramp-cache-data))
 
@@ -304,4 +313,5 @@ history."
 
 (provide 'tramp-cache)
 
+;; arch-tag: ee1739b7-7628-408c-9b96-d11a74b05d26
 ;;; tramp-cache.el ends here
