@@ -10,8 +10,8 @@
 
 ;; GNU Emacs is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 2, or (at your option)
-;; any later version.
+;; the Free Software Foundation; either version 3 of the License, or
+;; (at your option) any later version.
 
 ;; GNU Emacs is distributed in the hope that it will be useful,
 ;; but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -19,9 +19,8 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-;; Boston, MA 02110-1301, USA.
+;; along with GNU Emacs; see the file COPYING.  If not, see
+;; <http://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
@@ -264,6 +263,7 @@ Used instead of analyzing error codes of commands.")
     (make-symbolic-link . tramp-fish-handle-make-symbolic-link)
     (rename-file . tramp-fish-handle-rename-file)
     (set-file-modes . tramp-fish-handle-set-file-modes)
+    (set-file-times . tramp-fish-handle-set-file-times)
     (set-visited-file-modtime . ignore)
     (shell-command . tramp-handle-shell-command)
     (substitute-in-file-name . tramp-handle-substitute-in-file-name)
@@ -272,9 +272,8 @@ Used instead of analyzing error codes of commands.")
     (verify-visited-file-modtime . ignore)
     (write-region . tramp-fish-handle-write-region)
     (executable-find . tramp-fish-handle-executable-find)
-    (start-process . ignore)
-    (call-process . tramp-fish-handle-call-process)
-    (process-file . tramp-handle-process-file)
+    (start-file-process . ignore)
+    (process-file . tramp-fish-handle-process-file)
 )
   "Alist of handler functions for Tramp FISH method.
 Operations not mentioned here will be handled by the default Emacs primitives.")
@@ -301,7 +300,7 @@ pass to the OPERATION."
 
 (defun tramp-fish-handle-add-name-to-file
   (filename newname &optional ok-if-already-exists)
-  "Like `add-name-to-file' for tramp files."
+  "Like `add-name-to-file' for Tramp files."
   (unless (tramp-equal-remote filename newname)
     (with-parsed-tramp-file-name
 	(if (tramp-tramp-file-p filename) filename newname) nil
@@ -329,12 +328,12 @@ pass to the OPERATION."
 
 (defun tramp-fish-handle-copy-file
   (filename newname &optional ok-if-already-exists keep-date)
-  "Like `copy-file' for tramp files."
+  "Like `copy-file' for Tramp files."
   (tramp-fish-do-copy-or-rename-file
    'copy filename newname ok-if-already-exists keep-date))
 
 (defun tramp-fish-handle-delete-directory (directory)
-  "Like `delete-directory' for tramp files."
+  "Like `delete-directory' for Tramp files."
   (when (file-exists-p directory)
     (with-parsed-tramp-file-name
 	(directory-file-name (expand-file-name directory)) nil
@@ -342,7 +341,7 @@ pass to the OPERATION."
       (tramp-fish-send-command-and-check v (format "#RMD %s" localname)))))
 
 (defun tramp-fish-handle-delete-file (filename)
-  "Like `delete-file' for tramp files."
+  "Like `delete-file' for Tramp files."
   (when (file-exists-p filename)
     (with-parsed-tramp-file-name (expand-file-name filename) nil
       (tramp-flush-file-property v localname)
@@ -350,17 +349,17 @@ pass to the OPERATION."
 
 (defun tramp-fish-handle-directory-files-and-attributes
   (directory &optional full match nosort id-format)
-  "Like `directory-files-and-attributes' for tramp files."
+  "Like `directory-files-and-attributes' for Tramp files."
   (mapcar
    (lambda (x)
      ;; We cannot call `file-attributes' for backward compatibility reasons.
      ;; Its optional parameter ID-FORMAT is introduced with Emacs 22.
      (cons x (tramp-fish-handle-file-attributes
-	(if full x (concat (file-name-as-directory directory) x)) id-format)))
+	(if full x (expand-file-name x directory)) id-format)))
    (directory-files directory full match nosort)))
 
 (defun tramp-fish-handle-expand-file-name (name &optional dir)
-  "Like `expand-file-name' for tramp files."
+  "Like `expand-file-name' for Tramp files."
   ;; If DIR is not given, use DEFAULT-DIRECTORY or "/".
   (setq dir (or dir default-directory "/"))
   ;; Unless NAME is absolute, concat DIR and NAME.
@@ -384,7 +383,7 @@ pass to the OPERATION."
 	    (tramp-error
 	     v 'file-error "Tilde expansion not supported for %s" name))
 	  (setq uname
-	    (with-connection-property v "home-directory"
+	    (with-connection-property v uname
 	      (tramp-fish-send-command-and-check v "#PWD")
 	      (with-current-buffer (tramp-get-buffer v)
 		(goto-char (point-min))
@@ -409,13 +408,13 @@ pass to the OPERATION."
 				    (list localname)))))))))
 
 (defun tramp-fish-handle-file-attributes (filename &optional id-format)
-  "Like `file-attributes' for tramp files."
+  "Like `file-attributes' for Tramp files."
   (with-parsed-tramp-file-name (expand-file-name filename) nil
     (with-file-property v localname (format "file-attributes-%s" id-format)
       (cdr (car (tramp-fish-get-file-entries v localname nil))))))
 
 (defun tramp-fish-handle-file-directory-p (filename)
-  "Like `file-directory-p' for tramp files."
+  "Like `file-directory-p' for Tramp files."
   (let ((attributes (file-attributes filename)))
     (and attributes
 	 (or (string-match "d" (nth 8 attributes))
@@ -427,11 +426,11 @@ pass to the OPERATION."
 	 t)))
 
 (defun tramp-fish-handle-file-exists-p (filename)
-  "Like `file-exists-p' for tramp files."
+  "Like `file-exists-p' for Tramp files."
   (and (file-attributes filename) t))
 
 (defun tramp-fish-handle-file-executable-p (filename)
-  "Like `file-executable-p' for tramp files."
+  "Like `file-executable-p' for Tramp files."
   (with-parsed-tramp-file-name (expand-file-name filename) nil
     (with-file-property v localname "file-executable-p"
       (when (file-exists-p filename)
@@ -449,7 +448,7 @@ pass to the OPERATION."
 	      (char-equal (aref mode-chars 9) ?x)))))))
 
 (defun tramp-fish-handle-file-readable-p (filename)
-  "Like `file-readable-p' for tramp files."
+  "Like `file-readable-p' for Tramp files."
   (with-parsed-tramp-file-name (expand-file-name filename) nil
     (with-file-property v localname "file-readable-p"
       (when (file-exists-p filename)
@@ -467,7 +466,7 @@ pass to the OPERATION."
 	      (char-equal (aref mode-chars 7) ?r)))))))
 
 (defun tramp-fish-handle-file-writable-p (filename)
-  "Like `file-writable-p' for tramp files."
+  "Like `file-writable-p' for Tramp files."
   (with-parsed-tramp-file-name (expand-file-name filename) nil
     (with-file-property v localname "file-writable-p"
       (if (not (file-exists-p filename))
@@ -489,7 +488,7 @@ pass to the OPERATION."
 	      (char-equal (aref mode-chars 8) ?w)))))))
 
 (defun tramp-fish-handle-file-local-copy (filename)
-  "Like `file-local-copy' for tramp files."
+  "Like `file-local-copy' for Tramp files."
   (with-parsed-tramp-file-name (expand-file-name filename) nil
     (unless (file-exists-p filename)
       (tramp-error
@@ -507,7 +506,7 @@ pass to the OPERATION."
 ;; This function should return "foo/" for directories and "bar" for
 ;; files.
 (defun tramp-fish-handle-file-name-all-completions (filename directory)
-  "Like `file-name-all-completions' for tramp files."
+  "Like `file-name-all-completions' for Tramp files."
   (all-completions
    filename
    (with-parsed-tramp-file-name (expand-file-name directory) nil
@@ -525,7 +524,7 @@ pass to the OPERATION."
 	    entries)))))))
 
 (defun tramp-fish-handle-file-newer-than-file-p (file1 file2)
-  "Like `file-newer-than-file-p' for tramp files."
+  "Like `file-newer-than-file-p' for Tramp files."
   (cond
    ((not (file-exists-p file1)) nil)
    ((not (file-exists-p file2)) t)
@@ -534,7 +533,7 @@ pass to the OPERATION."
 
 (defun tramp-fish-handle-insert-directory
   (filename switches &optional wildcard full-directory-p)
-  "Like `insert-directory' for tramp files.
+  "Like `insert-directory' for Tramp files.
 WILDCARD and FULL-DIRECTORY-P are not handled."
   (setq filename (expand-file-name filename))
   (when (file-directory-p filename)
@@ -586,7 +585,7 @@ WILDCARD and FULL-DIRECTORY-P are not handled."
 
 (defun tramp-fish-handle-insert-file-contents
   (filename &optional visit beg end replace)
-  "Like `insert-file-contents' for tramp files."
+  "Like `insert-file-contents' for Tramp files."
   (barf-if-buffer-read-only)
   (when visit
     (setq buffer-file-name (expand-file-name filename))
@@ -615,10 +614,10 @@ WILDCARD and FULL-DIRECTORY-P are not handled."
 	(list (expand-file-name filename) size)))))
 
 (defun tramp-fish-handle-make-directory (dir &optional parents)
-  "Like `make-directory' for tramp files."
+  "Like `make-directory' for Tramp files."
   (setq dir (directory-file-name (expand-file-name dir)))
   (unless (file-name-absolute-p dir)
-    (setq dir (concat default-directory dir)))
+    (setq dir (expand-file-name dir default-directory)))
   (with-parsed-tramp-file-name dir nil
     (save-match-data
       (let ((ldir (file-name-directory dir)))
@@ -632,10 +631,10 @@ WILDCARD and FULL-DIRECTORY-P are not handled."
 	  (tramp-error v 'file-error "Couldn't make directory %s" dir))))))
 
 (defun tramp-fish-handle-make-directory-internal (directory)
-  "Like `make-directory-internal' for tramp files."
+  "Like `make-directory-internal' for Tramp files."
   (setq directory (directory-file-name (expand-file-name directory)))
   (unless (file-name-absolute-p directory)
-    (setq directory (concat default-directory directory)))
+    (setq directory (expand-file-name directory default-directory)))
   (when (file-directory-p (file-name-directory directory))
     (with-parsed-tramp-file-name directory nil
       (save-match-data
@@ -646,7 +645,7 @@ WILDCARD and FULL-DIRECTORY-P are not handled."
 
 (defun tramp-fish-handle-make-symbolic-link
   (filename linkname &optional ok-if-already-exists)
-  "Like `make-symbolic-link' for tramp files.
+  "Like `make-symbolic-link' for Tramp files.
 If LINKNAME is a non-Tramp file, it is used verbatim as the target of
 the symlink.  If LINKNAME is a Tramp file, only the localname component is
 used as the target of the symlink.
@@ -684,12 +683,12 @@ target of the symlink differ."
 
 (defun tramp-fish-handle-rename-file
   (filename newname &optional ok-if-already-exists)
-  "Like `rename-file' for tramp files."
+  "Like `rename-file' for Tramp files."
   (tramp-fish-do-copy-or-rename-file
    'rename filename newname ok-if-already-exists t))
 
 (defun tramp-fish-handle-set-file-modes (filename mode)
-  "Like `set-file-modes' for tramp files."
+  "Like `set-file-modes' for Tramp files."
   (with-parsed-tramp-file-name filename nil
     (tramp-flush-file-property v localname)
     (unless (tramp-fish-send-command-and-check
@@ -699,9 +698,18 @@ target of the symlink differ."
       (tramp-error
        v 'file-error "Error while changing file's mode %s" filename))))
 
+(defun tramp-fish-handle-set-file-times (filename &optional time)
+  "Like `set-file-times' for Tramp files."
+  (with-parsed-tramp-file-name filename nil
+    (let ((time (if (or (null time) (equal time '(0 0))) (current-time) time)))
+      (zerop (process-file
+	      "touch" nil nil nil "-t"
+	      (format-time-string "%Y%m%d%H%M.%S" time)
+	      (tramp-shell-quote-argument localname))))))
+
 (defun tramp-fish-handle-write-region
   (start end filename &optional append visit lockname confirm)
-  "Like `write-region' for tramp files."
+  "Like `write-region' for Tramp files."
   (setq filename (expand-file-name filename))
   (with-parsed-tramp-file-name filename nil
     ;; XEmacs takes a coding system as the seventh argument, not `confirm'
@@ -732,14 +740,14 @@ target of the symlink differ."
 (defun tramp-fish-handle-executable-find (command)
   "Like `executable-find' for Tramp files."
   (with-temp-buffer
-    (if (zerop (call-process "which" nil t nil command))
+    (if (zerop (process-file "which" nil t nil command))
 	(progn
 	  (goto-char (point-min))
 	  (buffer-substring (point-min) (tramp-line-end-position))))))
 
-(defun tramp-fish-handle-call-process
+(defun tramp-fish-handle-process-file
   (program &optional infile destination display &rest args)
-  "Like `call-process' for Tramp files."
+  "Like `process-file' for Tramp files."
   ;; The implementation is not complete yet.
   (when (and (numberp destination) (zerop destination))
     (error "Implementation does not handle immediate return"))
@@ -927,11 +935,8 @@ KEEP-DATE is non-nil, preserve the time stamp when copying."
 	       (tramp-shell-quote-argument v1-localname)
 	       (tramp-shell-quote-argument v2-localname)))))
   ;; KEEP-DATE handling.
-  (when keep-date
-    (let ((modtime (nth 5 (file-attributes filename))))
-      (when (and (not (null modtime))
-		 (not (equal modtime '(0 0))))
-	(tramp-touch newname modtime))))
+  (when (and keep-date (functionp 'set-file-times))
+    (apply 'set-file-times (list newname (nth 5 (file-attributes filename)))))
   ;; Set the mode.
   (set-file-modes newname (file-modes filename)))
 
@@ -943,7 +948,8 @@ Result is a list of (LOCALNAME LINK COUNT UID GID ATIME MTIME CTIME
 SIZE MODE WEIRD INODE DEVICE)."
   (block nil
     (with-current-buffer (tramp-get-buffer vec)
-      ;; #LIST does not work properly with trailing "/", at least in .fishsrv.pl
+      ;; #LIST does not work properly with trailing "/", at least in
+      ;; .fishsrv.pl.
       (when (string-match "/$" localname)
 	(setq localname (concat localname ".")))
 
@@ -975,7 +981,7 @@ SIZE MODE WEIRD INODE DEVICE)."
 	    ;; Add inode and device.
 	    (add-to-list
 	     'res (append item
-			  (list (tramp-get-inode (car item))
+			  (list (tramp-get-inode vec)
 				(tramp-get-device vec))))))
 
 	;; Read return code
@@ -1174,4 +1180,6 @@ Returns nil if there has been an error message."
 ;
 ;; * Evaluate the MIME information with #LIST or #STAT.
 ;
+
+;; arch-tag: a66df7df-5f29-42a7-a921-643ceb29db49
 ;;;; tramp-fish.el ends here
