@@ -10,8 +10,8 @@
 
 ;; GNU Emacs is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 2, or (at your option)
-;; any later version.
+;; the Free Software Foundation; either version 3 of the License, or
+;; (at your option) any later version.
 
 ;; GNU Emacs is distributed in the hope that it will be useful,
 ;; but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -19,9 +19,8 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-;; Boston, MA 02110-1301, USA.
+;; along with GNU Emacs; see the file COPYING.  If not, see
+;; <http://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
@@ -33,9 +32,7 @@
 (require 'tramp-cache)
 
 ;; Pacify byte-compiler
-(eval-when-compile
-  (require 'cl)
-  (require 'custom))
+(eval-when-compile (require 'custom))
 
 ;; Avoid byte-compiler warnings if the byte-compiler supports this.
 ;; Currently, XEmacs supports this.
@@ -79,8 +76,9 @@
   ;; `regexp-opt' not possible because of first string.
   (mapconcat
    'identity
-   '(;; Connection error
+   '(;; Connection error / timeout
      "Connection to \\S-+ failed"
+     "Read from server failed, maybe it closed the connection"
      ;; Samba
      "ERRDOS"
      "ERRSRV"
@@ -215,7 +213,7 @@ pass to the OPERATION."
 
 (defun tramp-smb-handle-copy-file
   (filename newname &optional ok-if-already-exists keep-date)
-  "Like `copy-file' for tramp files.
+  "Like `copy-file' for Tramp files.
 KEEP-DATE is not handled in case NEWNAME resides on an SMB server."
   (setq filename (expand-file-name filename)
 	newname (expand-file-name newname))
@@ -253,7 +251,7 @@ KEEP-DATE is not handled in case NEWNAME resides on an SMB server."
 	    (tramp-error v 'file-error "Cannot copy `%s'" filename)))))))
 
 (defun tramp-smb-handle-delete-directory (directory)
-  "Like `delete-directory' for tramp files."
+  "Like `delete-directory' for Tramp files."
   (setq directory (directory-file-name (expand-file-name directory)))
   (when (file-exists-p directory)
     (with-parsed-tramp-file-name directory nil
@@ -277,7 +275,7 @@ KEEP-DATE is not handled in case NEWNAME resides on an SMB server."
 	  (tramp-smb-send-command v (format "cd \\")))))))
 
 (defun tramp-smb-handle-delete-file (filename)
-  "Like `delete-file' for tramp files."
+  "Like `delete-file' for Tramp files."
   (setq filename (expand-file-name filename))
   (when (file-exists-p filename)
     (with-parsed-tramp-file-name filename nil
@@ -302,7 +300,7 @@ KEEP-DATE is not handled in case NEWNAME resides on an SMB server."
 
 (defun tramp-smb-handle-directory-files
   (directory &optional full match nosort)
-  "Like `directory-files' for tramp files."
+  "Like `directory-files' for Tramp files."
   (let ((result (mapcar 'directory-file-name
 			(file-name-all-completions "" directory))))
     ;; Discriminate with regexp
@@ -315,7 +313,7 @@ KEEP-DATE is not handled in case NEWNAME resides on an SMB server."
     (when full
       (setq result
 	    (mapcar
-	     (lambda (x) (concat (file-name-as-directory directory) x))
+	     (lambda (x) (expand-file-name x directory))
 	     result)))
     ;; Sort them if necessary
     (unless nosort (setq result (sort result 'string-lessp)))
@@ -324,17 +322,17 @@ KEEP-DATE is not handled in case NEWNAME resides on an SMB server."
 
 (defun tramp-smb-handle-directory-files-and-attributes
   (directory &optional full match nosort id-format)
-  "Like `directory-files-and-attributes' for tramp files."
+  "Like `directory-files-and-attributes' for Tramp files."
   (mapcar
    (lambda (x)
      ;; We cannot call `file-attributes' for backward compatibility reasons.
      ;; Its optional parameter ID-FORMAT is introduced with Emacs 22.
      (cons x (tramp-smb-handle-file-attributes
-	(if full x (concat (file-name-as-directory directory) x)) id-format)))
+	(if full x (expand-file-name x directory)) id-format)))
    (directory-files directory full match nosort)))
 
 (defun tramp-smb-handle-file-attributes (filename &optional id-format)
-  "Like `file-attributes' for tramp files."
+  "Like `file-attributes' for Tramp files."
   ;; Reading just the filename entry via "dir localname" is not
   ;; possible, because when filename is a directory, some smbclient
   ;; versions return the content of the directory, and other versions
@@ -348,7 +346,7 @@ KEEP-DATE is not handled in case NEWNAME resides on an SMB server."
 			 (assoc (file-name-nondirectory filename) entries)))
 	     (uid (if (and id-format (equal id-format 'string)) "nobody" -1))
 	     (gid (if (and id-format (equal id-format 'string)) "nogroup" -1))
-	     (inode (tramp-get-inode filename))
+	     (inode (tramp-get-inode v))
 	     (device (tramp-get-device v)))
 
         ;; Check result.
@@ -368,16 +366,16 @@ KEEP-DATE is not handled in case NEWNAME resides on an SMB server."
 		device))))))  ;11 file system number
 
 (defun tramp-smb-handle-file-directory-p (filename)
-  "Like `file-directory-p' for tramp files."
+  "Like `file-directory-p' for Tramp files."
   (and (file-exists-p filename)
        (eq ?d (aref (nth 8 (file-attributes filename)) 0))))
 
 (defun tramp-smb-handle-file-exists-p (filename)
-  "Like `file-exists-p' for tramp files."
+  "Like `file-exists-p' for Tramp files."
   (not (null (file-attributes filename))))
 
 (defun tramp-smb-handle-file-local-copy (filename)
-  "Like `file-local-copy' for tramp files."
+  "Like `file-local-copy' for Tramp files."
   (with-parsed-tramp-file-name filename nil
     (let ((file (tramp-smb-get-localname localname t))
 	  (tmpfil (tramp-make-temp-file filename)))
@@ -397,7 +395,7 @@ KEEP-DATE is not handled in case NEWNAME resides on an SMB server."
 ;; This function should return "foo/" for directories and "bar" for
 ;; files.
 (defun tramp-smb-handle-file-name-all-completions (filename directory)
-  "Like `file-name-all-completions' for tramp files."
+  "Like `file-name-all-completions' for Tramp files."
   (all-completions
    filename
    (with-parsed-tramp-file-name directory nil
@@ -413,7 +411,7 @@ KEEP-DATE is not handled in case NEWNAME resides on an SMB server."
 	    entries)))))))
 
 (defun tramp-smb-handle-file-newer-than-file-p (file1 file2)
-  "Like `file-newer-than-file-p' for tramp files."
+  "Like `file-newer-than-file-p' for Tramp files."
   (cond
    ((not (file-exists-p file1)) nil)
    ((not (file-exists-p file2)) t)
@@ -421,7 +419,7 @@ KEEP-DATE is not handled in case NEWNAME resides on an SMB server."
 			 (nth 5 (file-attributes file1))))))
 
 (defun tramp-smb-handle-file-writable-p (filename)
-  "Like `file-writable-p' for tramp files."
+  "Like `file-writable-p' for Tramp files."
   (if (file-exists-p filename)
       (string-match "w" (or (nth 8 (file-attributes filename)) ""))
     (let ((dir (file-name-directory filename)))
@@ -430,7 +428,7 @@ KEEP-DATE is not handled in case NEWNAME resides on an SMB server."
 
 (defun tramp-smb-handle-insert-directory
   (filename switches &optional wildcard full-directory-p)
-  "Like `insert-directory' for tramp files."
+  "Like `insert-directory' for Tramp files."
   (setq filename (expand-file-name filename))
   (when full-directory-p
     ;; Called from `dired-add-entry'.
@@ -501,10 +499,10 @@ KEEP-DATE is not handled in case NEWNAME resides on an SMB server."
 	   entries)))))
 
 (defun tramp-smb-handle-make-directory (dir &optional parents)
-  "Like `make-directory' for tramp files."
+  "Like `make-directory' for Tramp files."
   (setq dir (directory-file-name (expand-file-name dir)))
   (unless (file-name-absolute-p dir)
-    (setq dir (concat default-directory dir)))
+    (setq dir (expand-file-name dir default-directory)))
   (with-parsed-tramp-file-name dir nil
     (save-match-data
       (let* ((share (tramp-smb-get-share localname))
@@ -519,10 +517,10 @@ KEEP-DATE is not handled in case NEWNAME resides on an SMB server."
 	  (tramp-error v 'file-error "Couldn't make directory %s" dir))))))
 
 (defun tramp-smb-handle-make-directory-internal (directory)
-  "Like `make-directory-internal' for tramp files."
+  "Like `make-directory-internal' for Tramp files."
   (setq directory (directory-file-name (expand-file-name directory)))
   (unless (file-name-absolute-p directory)
-    (setq directory (concat default-directory directory)))
+    (setq directory (expand-file-name directory default-directory)))
   (with-parsed-tramp-file-name directory nil
     (save-match-data
       (let* ((file (tramp-smb-get-localname localname t)))
@@ -537,7 +535,7 @@ KEEP-DATE is not handled in case NEWNAME resides on an SMB server."
 
 (defun tramp-smb-handle-rename-file
   (filename newname &optional ok-if-already-exists)
-  "Like `rename-file' for tramp files."
+  "Like `rename-file' for Tramp files."
   (setq filename (expand-file-name filename)
 	newname (expand-file-name newname))
 
@@ -570,7 +568,7 @@ KEEP-DATE is not handled in case NEWNAME resides on an SMB server."
   (delete-file filename))
 
 (defun tramp-smb-handle-substitute-in-file-name (filename)
-  "Like `handle-substitute-in-file-name' for tramp files.
+  "Like `handle-substitute-in-file-name' for Tramp files.
 Catches errors for shares like \"C$/\", which are common in Microsoft Windows."
   (condition-case nil
       (tramp-run-real-handler 'substitute-in-file-name (list filename))
@@ -578,7 +576,7 @@ Catches errors for shares like \"C$/\", which are common in Microsoft Windows."
 
 (defun tramp-smb-handle-write-region
   (start end filename &optional append visit lockname confirm)
-  "Like `write-region' for tramp files."
+  "Like `write-region' for Tramp files."
   (setq filename (expand-file-name filename))
   (with-parsed-tramp-file-name filename nil
     (unless (eq append nil)
@@ -864,7 +862,22 @@ connection if a previous connection has died for some reason."
   (let* ((share (tramp-smb-get-share (tramp-file-name-localname vec)))
 	 (buf (tramp-get-buffer vec))
 	 (p (get-buffer-process buf)))
-    ;; Check whether it is still the same share
+
+    ;; If too much time has passed since last command was sent, look
+    ;; whether has been an error message; maybe due to connection timeout.
+    (with-current-buffer buf
+      (goto-char (point-min))
+      (when (and (> (tramp-time-diff
+		     (current-time)
+		     (tramp-get-connection-property
+		      p "last-cmd-time" '(0 0 0)))
+		    60)
+		 p (processp p) (memq (process-status p) '(run open))
+		 (re-search-forward tramp-smb-errors nil t))
+	(delete-process p)
+	(setq p nil)))
+
+    ;; Check whether it is still the same share.
     (unless
 	(and p (processp p) (memq (process-status p) '(run open))
 	     (string-equal
