@@ -10,8 +10,8 @@
 
 ;; This file is free software; you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 2, or (at your option)
-;; any later version.
+;; the Free Software Foundation; either version 3 of the License, or
+;; (at your option) any later version.
 
 ;; This file is distributed in the hope that it will be useful,
 ;; but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -19,9 +19,8 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to
-;; the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-;; Boston, MA 02110-1301, USA.
+;; along with GNU Emacs; see the file COPYING.  If not, see
+;; <http://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
@@ -30,7 +29,6 @@
 
 ;;; Code:
 
-(eval-when-compile (require 'cl))
 (require 'tramp)
 
 ;; Define a Tramp minor mode. It's intention is to redefine some keys
@@ -77,7 +75,7 @@ into account.  XEmacs menubar bindings are not changed by this."
 ;; Utility functions.
 
 ;; `executable-find', `start-process' and `call-process' have no file
-;; name handler yet.  The idea is that such a handler is called when
+;; name handler.  The idea is that such a handler is called when
 ;; `default-directory' matches a regexp in `file-name-handler-alist'.
 ;; This would allow to run commands on remote hosts.  The disadvantage
 ;; is, that commands which should run locally anyway, would also run
@@ -97,9 +95,13 @@ into account.  XEmacs menubar bindings are not changed by this."
 ;; assumed that the shell on a remote host is equal to the one of the
 ;; local host.
 
+;; With Emacs 23 this shouldn't be necessary any longer.  We check it
+;; via the existence of `start-file-process', which has introduced
+;; there.
+
 ;; Other open problems are `setenv'/`getenv'.
 
-(unless (tramp-exists-file-name-handler 'executable-find "ls")
+(unless (fboundp 'start-file-process)
   (defadvice executable-find
     (around tramp-advice-executable-find activate)
     "Invoke `tramp-handle-executable-find' for Tramp files."
@@ -120,16 +122,15 @@ into account.  XEmacs menubar bindings are not changed by this."
 		(t ad-do-it)))
       ad-do-it))
   (add-hook 'tramp-util-unload-hook
-	    '(lambda () (ad-unadvise 'executable-find))))
+	    '(lambda () (ad-unadvise 'executable-find)))
 
-(unless (tramp-exists-file-name-handler 'start-process "" nil "ls")
   (defadvice start-process
     (around tramp-advice-start-process activate)
     "Invoke `tramp-handle-start-process' for Tramp files."
     (if (eq (tramp-find-foreign-file-name-handler default-directory)
 	    'tramp-sh-file-name-handler)
 	(setq ad-return-value
-	      (apply 'tramp-handle-start-process (ad-get-args 0)))
+	      (apply 'tramp-handle-start-file-process (ad-get-args 0)))
       ad-do-it))
   (add-hook 'tramp-util-unload-hook
 	    '(lambda () (ad-unadvise 'start-process)))
@@ -146,9 +147,8 @@ into account.  XEmacs menubar bindings are not changed by this."
 	    ad-do-it))
       ad-do-it))
   (add-hook 'tramp-util-unload-hook
-	    '(lambda () (ad-unadvise 'start-process-shell-command))))
+	    '(lambda () (ad-unadvise 'start-process-shell-command)))
 
-(unless (tramp-exists-file-name-handler 'call-process "ls")
   (defadvice call-process
     (around tramp-advice-call-process activate)
     "Invoke `tramp-handle-call-process' for Tramp files."
@@ -195,16 +195,20 @@ into account.  XEmacs menubar bindings are not changed by this."
   (unless (tramp-exists-file-name-handler 'file-remote-p "/")
     ;; XEmacs 21
     (defadvice file-remote-p
-      (around tramp-advice-file-remote-p (filename) activate)
+      (around tramp-advice-file-remote-p
+	      (filename &optional identification connected) activate)
       "Invoke `tramp-handle-file-remote-p' for Tramp files."
       (if (eq (tramp-find-foreign-file-name-handler (expand-file-name filename))
 	      'tramp-sh-file-name-handler)
 	  (setq ad-return-value
-		(tramp-handle-file-remote-p filename))
+		(tramp-handle-file-remote-p filename identification connected))
 	ad-do-it))
     (add-hook 'tramp-util-unload-hook
 	      '(lambda () (ad-unadvise 'file-remote-p)))))
 
+(if (not (fboundp 'set-file-times))
+    (defalias 'set-file-times (symbol-function 'tramp-handle-set-file-times)))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 ;; In Emacs 21, `dired-insert-directory' calls
@@ -301,7 +305,7 @@ into account.  XEmacs menubar bindings are not changed by this."
     (eval
      `(add-hook
        'tramp-util-unload-hook
-       '(lambda () (ad-unadvise ,x)))))
+       '(lambda () (ad-unadvise (quote ,x))))))
 
  '(grep grep-find grep-tree grep-process-setup))
 
@@ -354,7 +358,7 @@ If it is an absolute file name, and not a remote one, prepend the remote part."
 	      ;; It is already expanded.
 	      filename
 	    ;; Prefix the Tramp remote file name.
-	    (concat (file-remote-p default-directory) filename)))
+	    (expand-file-name filename (file-remote-p default-directory))))
 
     ;; Emacs 22 uses `gud-file-name' which we should do as well.
     ;; `gud-<MINOR-MODE>-directories' must be Tramp file names.
@@ -398,7 +402,8 @@ Works only for relative file names and Tramp file names."
     (eval
      `(add-hook
        'tramp-util-unload-hook
-       '(lambda () (ad-unadvise ,(intern (format "gud-%s-find-file" x))))))
+       '(lambda ()
+	  (ad-unadvise (quote ,(intern (format "gud-%s-find-file" x)))))))
 
     ;; Arguments shall be trimmed to local file names.
     (eval
@@ -415,8 +420,8 @@ Works only for relative file names and Tramp file names."
     (eval
      `(add-hook
        'tramp-util-unload-hook
-       '(lambda () (ad-unadvise
-		    ,(intern (format "gud-%s-massage-args" x)))))))
+       '(lambda ()
+	  (ad-unadvise (quote ,(intern (format "gud-%s-massage-args" x))))))))
 
  ;; So far, I've tested only gdb and perldb.
  ;; (X)Emacs
@@ -428,7 +433,23 @@ Works only for relative file names and Tramp file names."
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
-;; several packages expect that the processes run locally.
+;; Files to be compared in ediff.el shall be local.
+
+(defadvice ediff-exec-process
+  (before tramp-advice-before-ediff-exec-process
+	  (program buffer synch options &rest files) activate)
+  "Make FILES local."
+  (setq files
+	(mapcar
+	 '(lambda (x) (when (stringp x) (or (file-local-copy x) x))) files)))
+(eval
+ `(add-hook
+   'tramp-util-unload-hook
+   '(lambda () (ad-unadvise 'ediff-exec-process))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+;; Several packages expect that the processes run locally.
 
 (mapcar
  '(lambda (x)
@@ -437,14 +458,12 @@ Works only for relative file names and Tramp file names."
 	(around ,(intern (format "tramp-advice-%s" x)) activate)
 	,(format "Run `%s on a local default directory." x)
 	(let ((default-directory
-		(if (file-remote-p default-directory)
-		    (expand-file-name "~/" "/")
-		  default-directory)))
+		(unhandled-file-name-directory default-directory)))
 	  ad-do-it)))
     (eval
      `(add-hook
        'tramp-util-unload-hook
-       '(lambda () (ad-unadvise ,x)))))
+       '(lambda () (ad-unadvise (quote ,x))))))
 
  '(browse-url-default-windows-browser
    browse-url-default-macosx-browser browse-url-netscape
